@@ -1,7 +1,7 @@
 # Author: David Goodger
 # Contact: goodger@users.sourceforge.net
-# Revision: $Revision: 1.55 $
-# Date: $Date: 2004/04/02 02:29:26 $
+# Revision: $Revision: 1.78 $
+# Date: $Date: 2004/09/30 03:52:14 $
 # Copyright: This module has been placed in the public domain.
 
 """
@@ -18,10 +18,11 @@ Exports the following classes:
 Also exports the following functions:
 
 * Option callbacks: `store_multiple`, `read_config_file`.
-* Setting validators (see `OptionParser.validators`: `validate_encoding`,
-  `validate_encoding_error_handler`, `validate_encoding_and_error_handler`,
-  `validate_boolean`, `validate_threshold`,
-  `validate_colon_separated_string_list`.
+* Setting validators: `validate_encoding`,
+  `validate_encoding_error_handler`,
+  `validate_encoding_and_error_handler`, `validate_boolean`,
+  `validate_threshold`, `validate_colon_separated_string_list`,
+  `validate_dependency_file`.
 * `make_paths_absolute`.
 """
 
@@ -37,13 +38,13 @@ import ConfigParser as CP
 import codecs
 import docutils
 import optparse
-from optparse import Values, SUPPRESS_HELP
+from optparse import SUPPRESS_HELP
 
 
 def store_multiple(option, opt, value, parser, *args, **kwargs):
     """
     Store multiple values in `parser.values`.  (Option callback.)
-    
+
     Store `None` for each attribute named in `args`, and store the value for
     each key (attribute name) in `kwargs`.
     """
@@ -77,10 +78,10 @@ def validate_encoding_error_handler(setting, value, option_parser,
     try:
         codecs.lookup_error(value)
     except AttributeError:              # prior to Python 2.3
-        if value not in ('strict', 'ignore', 'replace'):
+        if value not in ('strict', 'ignore', 'replace', 'xmlcharrefreplace'):
             raise (LookupError(
                 'unknown encoding error handler: "%s" (choices: '
-                '"strict", "ignore", or "replace")' % value),
+                '"strict", "ignore", "replace", or "xmlcharrefreplace")' % value),
                    None, sys.exc_info()[2])
     except LookupError:
         raise (LookupError(
@@ -143,6 +144,22 @@ def validate_colon_separated_string_list(
         value.extend(last.split(':'))
     return value
 
+def validate_url_trailing_slash(
+    setting, value, option_parser, config_parser=None, config_section=None):
+    if not value:
+        return './'
+    elif value.endswith('/'):
+        return value
+    else:
+        return value + '/'
+
+def validate_dependency_file(
+    setting, value, option_parser, config_parser=None, config_section=None):
+    try:
+        return docutils.utils.DependencyList(value)
+    except IOError:
+        return docutils.utils.DependencyList(None)
+
 def make_paths_absolute(pathdict, keys, base_path=None):
     """
     Interpret filesystem path settings relative to the `base_path` given.
@@ -173,6 +190,13 @@ class Values(optparse.Values):
     Works in conjunction with the `OptionParser.lists` instance attribute.
     """
 
+    def __init__(self, *args, **kwargs):
+        optparse.Values.__init__(self, *args, **kwargs)
+        if (not hasattr(self, 'record_dependencies')
+            or self.record_dependencies is None):
+            # Set up dependency list, in case it is needed.
+            self.record_dependencies = docutils.utils.DependencyList()
+
     def update(self, other_dict, option_parser):
         if isinstance(other_dict, Values):
             other_dict = other_dict.__dict__
@@ -188,25 +212,29 @@ class Values(optparse.Values):
 
 class Option(optparse.Option):
 
+    ATTRS = optparse.Option.ATTRS + ['validator', 'overrides']
+
     def process(self, opt, value, values, parser):
         """
-        Call the validator function on applicable settings.
+        Call the validator function on applicable settings and
+        evaluate the 'overrides' option.
         Extends `optparse.Option.process`.
         """
         result = optparse.Option.process(self, opt, value, values, parser)
         setting = self.dest
         if setting:
-            value = getattr(values, setting)
-            validator = parser.validators.get(setting)
-            if validator:
+            if self.validator:
+                value = getattr(values, setting)
                 try:
-                    new_value = validator(setting, value, parser)
+                    new_value = self.validator(setting, value, parser)
                 except Exception, error:
                     raise (optparse.OptionValueError(
                         'Error in option "%s":\n    %s: %s'
                         % (opt, error.__class__.__name__, error)),
                            None, sys.exc_info()[2])
                 setattr(values, setting, new_value)
+            if self.overrides:
+                setattr(values, self.overrides, None)
         return result
 
 
@@ -311,8 +339,9 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
           ['--quiet', '-q'], {'action': 'store_const', 'const': 'none',
                               'dest': 'report_level'}),
          ('Set the threshold (<level>) at or above which system messages are '
-          'converted to exceptions, halting execution immediately.  Levels '
-          'as in --report.  Default is 4 (severe).',
+          'converted to exceptions, halting execution immediately by '
+          'exiting (or propagating the exception if --traceback set).  '
+          'Levels as in --report.  Default is 4 (severe).',
           ['--halt'], {'choices': threshold_choices, 'dest': 'halt_level',
                        'default': 4, 'metavar': '<level>',
                        'validator': validate_threshold}),
@@ -323,9 +352,10 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
           'system messages (at or above <level>) were generated.  Levels as '
           'in --report.  Default is 5 (disabled).  Exit status is the maximum '
           'system message level plus 10 (11 for INFO, etc.).',
-          ['--exit'], {'choices': threshold_choices, 'dest': 'exit_level',
-                       'default': 5, 'metavar': '<level>',
-                       'validator': validate_threshold}),
+          ['--exit-status'], {'choices': threshold_choices,
+                              'dest': 'exit_status_level',
+                              'default': 5, 'metavar': '<level>',
+                              'validator': validate_threshold}),
          ('Report debug-level system messages and generate diagnostic output.',
           ['--debug'], {'action': 'store_true', 'validator': validate_boolean}),
          ('Do not report debug-level system messages or generate diagnostic '
@@ -333,33 +363,53 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
           ['--no-debug'], {'action': 'store_false', 'dest': 'debug'}),
          ('Send the output of system messages (warnings) to <file>.',
           ['--warnings'], {'dest': 'warning_stream', 'metavar': '<file>'}),
-         ('Enable Python tracebacks when an error occurs.',
+         ('Enable Python tracebacks when halt-level system messages and '
+          'other exceptions occur.  Useful for debugging, and essential for '
+          'issue reports.',
           ['--traceback'], {'action': 'store_true', 'default': None,
                             'validator': validate_boolean}),
          ('Disable Python tracebacks when errors occur; report just the error '
           'instead.  This is the default.',
           ['--no-traceback'], {'dest': 'traceback', 'action': 'store_false'}),
-         ('Specify the encoding of input text.  Default is locale-dependent.',
+         ('Specify the encoding of input text.  Default is locale-dependent.  '
+          'Optionally also specify the error handler for undecodable '
+          'characters, after a colon (":"); default is "strict".  (See '
+          '"--intput-encoding-error-handler".)',
           ['--input-encoding', '-i'],
-          {'metavar': '<name>', 'validator': validate_encoding}),
+          {'metavar': '<name[:handler]>',
+           'validator': validate_encoding_and_error_handler}),
+         ('Specify the error handler for undecodable characters in '
+          'the input.  Acceptable values include "strict", "ignore", and '
+          '"replace".  Default is "strict".  '
+          'Usually specified as part of --input-encoding.',
+          ['--input-encoding-error-handler'],
+          {'default': 'strict', 'validator': validate_encoding_error_handler}),
          ('Specify the text encoding for output.  Default is UTF-8.  '
-          'Optionally also specify the encoding error handler for unencodable '
-          'characters (see "--error-encoding"); default is "strict".',
+          'Optionally also specify the error handler for unencodable '
+          'characters, after a colon (":"); default is "strict".  (See '
+          '"--output-encoding-error-handler".)',
           ['--output-encoding', '-o'],
           {'metavar': '<name[:handler]>', 'default': 'utf-8',
            'validator': validate_encoding_and_error_handler}),
-         (SUPPRESS_HELP,                # usually handled by --output-encoding
+         ('Specify the error handler for unencodable characters in '
+          'the output.  Acceptable values include "strict", "ignore", '
+          '"replace", "xmlcharrefreplace", and '
+          '"backslashreplace" (in Python 2.3+).  Default is "strict".  '
+          'Usually specified as part of --output-encoding.',
           ['--output-encoding-error-handler'],
           {'default': 'strict', 'validator': validate_encoding_error_handler}),
          ('Specify the text encoding for error output.  Default is ASCII.  '
-          'Optionally also specify the encoding error handler for unencodable '
-          'characters, after a colon (":").  Acceptable values are the same '
-          'as for the "error" parameter of Python\'s ``encode`` string '
-          'method.  Default is "%s".' % default_error_encoding_error_handler,
+          'Optionally also specify the error handler for unencodable '
+          'characters, after a colon (":"); default is "%s".  (See '
+          '"--output-encoding-error-handler".)'
+          % default_error_encoding_error_handler,
           ['--error-encoding', '-e'],
           {'metavar': '<name[:handler]>', 'default': 'ascii',
            'validator': validate_encoding_and_error_handler}),
-         (SUPPRESS_HELP,                # usually handled by --error-encoding
+         ('Specify the error handler for unencodable characters in '
+          'error output.  See --output-encoding-error-handler for acceptable '
+          'values.  Default is "%s".  Usually specified as part of '
+          '--error-encoding.' % default_error_encoding_error_handler,
           ['--error-encoding-error-handler'],
           {'default': default_error_encoding_error_handler,
            'validator': validate_encoding_error_handler}),
@@ -367,6 +417,11 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
           '  Default is "en" (English).',
           ['--language', '-l'], {'dest': 'language_code', 'default': 'en',
                                  'metavar': '<name>'}),
+         ('Write dependencies (caused e.g. by file inclusions) to '
+          '<file>.  Useful in conjunction with programs like "make".',
+          ['--record-dependencies'],
+          {'metavar': '<file>', 'validator': validate_dependency_file,
+           'default': None}),           # default set in Values class
          ('Read configuration settings from <file>, if it exists.',
           ['--config'], {'metavar': '<file>', 'type': 'string',
                          'action': 'callback', 'callback': read_config_file}),
@@ -381,12 +436,16 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
          (SUPPRESS_HELP, ['--dump-pseudo-xml'], {'action': 'store_true'}),
          (SUPPRESS_HELP, ['--expose-internal-attribute'],
           {'action': 'append', 'dest': 'expose_internals',
-           'validator': validate_colon_separated_string_list}),))
+           'validator': validate_colon_separated_string_list}),
+         (SUPPRESS_HELP, ['--strict-visitor'], {'action': 'store_true'}),
+         ))
     """Runtime settings and command-line options common to all Docutils front
     ends.  Setting specs specific to individual Docutils components are also
     used (see `populate_from_components()`)."""
 
-    settings_defaults = {'_disable_config': None}
+    settings_defaults = {'_disable_config': None,
+                         '_source': None,
+                         '_destination': None}
     """Defaults for settings that don't have command-line option equivalents."""
 
     relative_path_settings = ('warning_stream',)
@@ -403,14 +462,6 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
         ``.settings_spec`` attribute.  `defaults` is a mapping of setting
         default overrides.
         """
-        self.validators = {}
-        """{setting: validation function} mapping, used by `validate_options`.
-        Validation functions take three or five parameters: setting name,
-        value, an `OptionParser` (``self``), and a `ConfigParser` and config
-        file section if activated from a config file.  They return a (possibly
-        modified) value, or raise an exception.  Populated from the "validator"
-        keyword argument dictionary entries of components' ``settings_spec``
-        attribute."""
 
         self.lists = {}
         """Set of list-type settings."""
@@ -425,16 +476,13 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
         self.relative_path_settings = list(self.relative_path_settings)
         self.components = (self,) + tuple(components)
         self.populate_from_components(self.components)
-        defaults = defaults or {}
+        self.set_defaults(**(defaults or {}))
         if read_config_files and not self.defaults['_disable_config']:
             try:
                 config_settings = self.get_standard_config_settings()
             except ValueError, error:
                 self.error(error)
-            defaults.update(config_settings.__dict__)
-        # Internal settings with no defaults from settings specifications;
-        # initialize manually:
-        self.set_defaults(_source=None, _destination=None, **defaults)
+            self.set_defaults(**config_settings.__dict__)
 
     def populate_from_components(self, components):
         """
@@ -446,11 +494,10 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
         for component in components:
             if component is None:
                 continue
-            i = 0
             settings_spec = component.settings_spec
             self.relative_path_settings.extend(
                 component.relative_path_settings)
-            while i < len(settings_spec):
+            for i in range(0, len(settings_spec), 3):
                 title, description, option_spec = settings_spec[i:i+3]
                 if title:
                     group = optparse.OptionGroup(self, title, description)
@@ -458,21 +505,12 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
                 else:
                     group = self        # single options
                 for (help_text, option_strings, kwargs) in option_spec:
-                    kwargs = kwargs.copy() # to be modified, locally only
-                    if kwargs.has_key('validator'):
-                        validator = kwargs['validator']
-                        del kwargs['validator']
-                    else:
-                        validator = None
                     option = group.add_option(help=help_text, *option_strings,
                                               **kwargs)
-                    if validator:
-                        self.validators[option.dest] = validator
                     if kwargs.get('action') == 'append':
                         self.lists[option.dest] = 1
                 if component.settings_defaults:
                     self.defaults.update(component.settings_defaults)
-                i += 3
         for component in components:
             if component and component.settings_default_overrides:
                 self.defaults.update(component.settings_default_overrides)
@@ -539,6 +577,22 @@ class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
         """Needed to get custom `Values` instances."""
         return Values(self.defaults)
 
+    def get_option_by_dest(self, dest):
+        """
+        Get an option by its dest.
+
+        If you're supplying a dest which is shared by several options,
+        it is undefined which option of those is returned.
+
+        A KeyError is raised if there is no option with the supplied
+        dest.
+        """
+        for group in self.option_groups + [self]:
+            for option in group.option_list:
+                if option.dest == dest:
+                    return option
+        raise KeyError('No option with dest == %r.' % dest)
+
 
 class ConfigParser(CP.ConfigParser):
 
@@ -552,8 +606,8 @@ class ConfigParser(CP.ConfigParser):
     old_warning = """
 The "[option]" section is deprecated.  Support for old-format configuration
 files may be removed in a future Docutils release.  Please revise your
-configuration files.  See <http://docutils.sf.net/docs/config.html>, section
-"Old-Format Configuration Files".
+configuration files.  See <http://docutils.sf.net/docs/user/config.html>,
+section "Old-Format Configuration Files".
 """
 
     def read(self, filenames, option_parser):
@@ -584,14 +638,20 @@ configuration files.  See <http://docutils.sf.net/docs/config.html>, section
         self.remove_section('options')
 
     def validate_settings(self, filename, option_parser):
-        """Call the validator function on all applicable settings."""
+        """
+        Call the validator function and implement overrides on all applicable
+        settings.
+        """
         for section in self.sections():
             for setting in self.options(section):
-                validator = option_parser.validators.get(setting)
-                if validator:
+                try:
+                    option = option_parser.get_option_by_dest(setting)
+                except KeyError:
+                    continue
+                if option.validator:
                     value = self.get(section, setting, raw=1)
                     try:
-                        new_value = validator(
+                        new_value = option.validator(
                             setting, value, option_parser,
                             config_parser=self, config_section=section)
                     except Exception, error:
@@ -601,6 +661,8 @@ configuration files.  See <http://docutils.sf.net/docs/config.html>, section
                             % (filename, section, error.__class__.__name__,
                                error, setting, value)), None, sys.exc_info()[2])
                     self.set(section, setting, new_value)
+                if option.overrides:
+                    self.set(section, option.overrides, None)
 
     def optionxform(self, optionstr):
         """
