@@ -1,7 +1,7 @@
 # Author: David Goodger
 # Contact: goodger@users.sourceforge.net
-# Revision: $Revision: 1.54 $
-# Date: $Date: 2004/04/09 22:49:02 $
+# Revision: $Revision: 1.68 $
+# Date: $Date: 2004/11/09 11:17:36 $
 # Copyright: This module has been placed in the public domain.
 
 """
@@ -18,7 +18,7 @@ is represented by abstract base classes (`Root`, `Structural`, `Body`,
 ``isinstance(node, base_class)`` to determine the position of the node in the
 hierarchy.
 
-.. _DTD: http://docutils.sourceforge.net/spec/docutils.dtd
+.. _DTD: http://docutils.sourceforge.net/docs/ref/docutils.dtd
 """
 
 __docformat__ = 'reStructuredText'
@@ -69,7 +69,11 @@ class Node:
         return self._dom_node(domroot)
 
     def pformat(self, indent='    ', level=0):
-        """Return an indented pseudo-XML representation, for test purposes."""
+        """
+        Return an indented pseudo-XML representation, for test purposes.
+
+        Override in subclasses.
+        """
         raise NotImplementedError
 
     def copy(self):
@@ -87,12 +91,11 @@ class Node:
 
     def walk(self, visitor):
         """
-        Traverse a tree of `Node` objects, calling ``visit_...`` methods of
-        `visitor` when entering each node. If there is no
-        ``visit_particular_node`` method for a node of type
-        ``particular_node``, the ``unknown_visit`` method is called.  (The
-        `walkabout()` method is similar, except it also calls ``depart_...``
-        methods before exiting each node.)
+        Traverse a tree of `Node` objects, calling the
+        `dispatch_visit()` method of `visitor` when entering each
+        node.  (The `walkabout()` method is similar, except it also
+        calls the `dispatch_departure()` method before exiting each
+        node.)
 
         This tree traversal supports limited in-place tree
         modifications.  Replacing one node with one or more nodes is
@@ -100,18 +103,18 @@ class Node:
         or replaced occurs after the current node, the old node will
         still be traversed, and any new nodes will not.
 
-        Within ``visit_...`` methods (and ``depart_...`` methods for
+        Within ``visit`` methods (and ``depart`` methods for 
         `walkabout()`), `TreePruningException` subclasses may be raised
         (`SkipChildren`, `SkipSiblings`, `SkipNode`, `SkipDeparture`).
 
         Parameter `visitor`: A `NodeVisitor` object, containing a
-        ``visit_...`` method for each `Node` subclass encountered.
+        ``visit`` implementation for each `Node` subclass encountered.
         """
-        name = 'visit_' + self.__class__.__name__
-        method = getattr(visitor, name, visitor.unknown_visit)
-        visitor.document.reporter.debug(name, category='nodes.Node.walk')
+        visitor.document.reporter.debug(
+            'calling dispatch_visit for %s' % self.__class__.__name__,
+            category='nodes.Node.walk')
         try:
-            method(self)
+            visitor.dispatch_visit(self)
         except (SkipChildren, SkipNode):
             return
         except SkipDeparture:           # not applicable; ignore
@@ -125,21 +128,21 @@ class Node:
 
     def walkabout(self, visitor):
         """
-        Perform a tree traversal similarly to `Node.walk()` (which see),
-        except also call ``depart_...`` methods before exiting each node. If
-        there is no ``depart_particular_node`` method for a node of type
-        ``particular_node``, the ``unknown_departure`` method is called.
+        Perform a tree traversal similarly to `Node.walk()` (which
+        see), except also call the `dispatch_departure()` method
+        before exiting each node.
 
-        Parameter `visitor`: A `NodeVisitor` object, containing ``visit_...``
-        and ``depart_...`` methods for each `Node` subclass encountered.
+        Parameter `visitor`: A `NodeVisitor` object, containing a
+        ``visit`` and ``depart`` implementation for each `Node`
+        subclass encountered.
         """
         call_depart = 1
-        name = 'visit_' + self.__class__.__name__
-        method = getattr(visitor, name, visitor.unknown_visit)
-        visitor.document.reporter.debug(name, category='nodes.Node.walkabout')
+        visitor.document.reporter.debug(
+            'calling dispatch_visit for %s' % self.__class__.__name__,
+            category='nodes.Node.walkabout')
         try:
             try:
-                method(self)
+                visitor.dispatch_visit(self)
             except SkipNode:
                 return
             except SkipDeparture:
@@ -153,11 +156,10 @@ class Node:
         except SkipChildren:
             pass
         if call_depart:
-            name = 'depart_' + self.__class__.__name__
-            method = getattr(visitor, name, visitor.unknown_departure)
             visitor.document.reporter.debug(
-                  name, category='nodes.Node.walkabout')
-            method(self)
+                'calling dispatch_departure for %s' % self.__class__.__name__,
+                category='nodes.Node.walkabout')
+            visitor.dispatch_departure(self)
 
 
 class Text(Node, UserString):
@@ -589,7 +591,7 @@ class Admonition(Body): pass
 class Special(Body):
     """Special internal body elements."""
 
-class Invisible:
+class Invisible(PreBibliographic):
     """Internal elements that don't appear in output."""
 
 class Part: pass
@@ -946,7 +948,6 @@ class rubric(Titular, TextElement): pass
 # ========================
 
 class docinfo(Bibliographic, Element): pass
-class info(Bibliographic, Element): pass
 class author(Bibliographic, TextElement): pass
 class authors(Bibliographic, Element): pass
 class organization(Bibliographic, TextElement): pass
@@ -1014,6 +1015,7 @@ class transition(Structural, Element): pass
 # ===============
 
 class paragraph(General, TextElement): pass
+class compound(General, Element): pass
 class bullet_list(Sequential, Element): pass
 class enumerated_list(Sequential, Element): pass
 class list_item(Part, Element): pass
@@ -1056,7 +1058,14 @@ class option_string(Part, TextElement): pass
 class description(Part, Element): pass
 class literal_block(General, FixedTextElement): pass
 class doctest_block(General, FixedTextElement): pass
-class line_block(General, FixedTextElement): pass
+class line_block(General, Element): pass
+
+
+class line(General, TextElement):
+
+    indent = None
+
+
 class block_quote(General, Element): pass
 class attribution(Part, TextElement): pass
 class attention(Admonition, Element): pass
@@ -1069,7 +1078,7 @@ class tip(Admonition, Element): pass
 class hint(Admonition, Element): pass
 class warning(Admonition, Element): pass
 class admonition(Admonition, Element): pass
-class comment(Special, Invisible, PreBibliographic, FixedTextElement): pass
+class comment(Special, Invisible, FixedTextElement): pass
 class substitution_definition(Special, Invisible, TextElement): pass
 class target(Special, Invisible, Inline, TextElement, Targetable): pass
 class footnote(General, Element, Labeled, BackLinkable): pass
@@ -1105,7 +1114,7 @@ class system_message(Special, PreBibliographic, Element, BackLinkable):
                                        self['level'], Element.astext(self))
 
 
-class pending(Special, Invisible, PreBibliographic, Element):
+class pending(Special, Invisible, Element):
 
     """
     The "pending" element is used to encapsulate a pending operation: the
@@ -1123,8 +1132,8 @@ class pending(Special, Invisible, PreBibliographic, Element):
 
     But the "contents" directive can't do its work until the entire document
     has been parsed and possibly transformed to some extent.  So the directive
-    code leaves a placeholder behind that will trigger the second phase of the
-    its processing, something like this::
+    code leaves a placeholder behind that will trigger the second phase of its
+    processing, something like this::
 
         <pending ...public attributes...> + internal attributes
 
@@ -1171,7 +1180,7 @@ class pending(Special, Invisible, PreBibliographic, Element):
 
     def copy(self):
         return self.__class__(self.transform, self.details, self.rawsource,
-                              **self.attribuates)
+                              **self.attributes)
 
 
 class raw(Special, Inline, PreBibliographic, FixedTextElement):
@@ -1222,7 +1231,7 @@ node_class_names = """
         authors
     block_quote bullet_list
     caption caution citation citation_reference classifier colspec comment
-        contact copyright
+        compound contact copyright
     danger date decoration definition definition_list definition_list_item
         description docinfo doctest_block document
     emphasis entry enumerated_list error
@@ -1230,8 +1239,8 @@ node_class_names = """
         footnote footnote_reference
     generated
     header hint
-    image important info inline
-    label legend line_block list_item literal literal_block
+    image important inline
+    label legend line line_block list_item literal literal_block
     note
     option option_argument option_group option_list option_list_item
         option_string organization
@@ -1249,14 +1258,17 @@ node_class_names = """
 class NodeVisitor:
 
     """
-    "Visitor" pattern [GoF95]_ abstract superclass implementation for document
-    tree traversals.
+    "Visitor" pattern [GoF95]_ abstract superclass implementation for
+    document tree traversals.
 
-    Each node class has corresponding methods, doing nothing by default;
-    override individual methods for specific and useful behaviour.  The
-    "``visit_`` + node class name" method is called by `Node.walk()` upon
-    entering a node.  `Node.walkabout()` also calls the "``depart_`` + node
-    class name" method before exiting a node.
+    Each node class has corresponding methods, doing nothing by
+    default; override individual methods for specific and useful
+    behaviour.  The `dispatch_visit()` method is called by
+    `Node.walk()` upon entering a node.  `Node.walkabout()` also calls
+    the `dispatch_departure()` method before exiting a node.
+
+    The dispatch methods call "``visit_`` + node class name" or
+    "``depart_`` + node class name", resp.
 
     This is a base class for visitors whose ``visit_...`` & ``depart_...``
     methods should be implemented for *all* node types encountered (such as
@@ -1272,17 +1284,56 @@ class NodeVisitor:
        1995.
     """
 
+    optional = ()
+    """
+    Tuple containing node class names (as strings).
+
+    No exception will be raised if writers do not implement visit
+    or departure functions for these node classes.
+
+    Used to ensure transitional compatibility with existing 3rd-party writers.
+    """
+
     def __init__(self, document):
         self.document = document
 
+    def dispatch_visit(self, node):
+        """
+        Call self."``visit_`` + node class name" with `node` as
+        parameter.  If the ``visit_...`` method does not exist, call
+        self.unknown_visit.
+        """
+        node_name = node.__class__.__name__
+        method = getattr(self, 'visit_' + node_name, self.unknown_visit)
+        self.document.reporter.debug(
+            'calling %s for %s' % (method.__name__, node_name),
+            category='nodes.NodeVisitor.dispatch_visit')
+        return method(node)
+
+    def dispatch_departure(self, node):
+        """
+        Call self."``depart_`` + node class name" with `node` as
+        parameter.  If the ``depart_...`` method does not exist, call
+        self.unknown_departure.
+        """
+        node_name = node.__class__.__name__
+        method = getattr(self, 'depart_' + node_name, self.unknown_departure)
+        self.document.reporter.debug(
+            'calling %s for %s' % (method.__name__, node_name),
+            category='nodes.NodeVisitor.dispatch_departure')
+        return method(node)
+
     def unknown_visit(self, node):
         """
         Called when entering unknown `Node` types.
 
         Raise an exception unless overridden.
         """
-        raise NotImplementedError('%s visiting unknown node type: %s'
-                                  % (self.__class__, node.__class__.__name__))
+        if  (node.document.settings.strict_visitor
+             or node.__class__.__name__ not in self.optional):
+            raise NotImplementedError(
+                '%s visiting unknown node type: %s'
+                % (self.__class__, node.__class__.__name__))
 
     def unknown_departure(self, node):
         """
@@ -1290,8 +1341,11 @@ class NodeVisitor:
 
         Raise exception unless overridden.
         """
-        raise NotImplementedError('%s departing unknown node type: %s'
-                                  % (self.__class__, node.__class__.__name__))
+        if  (node.document.settings.strict_visitor
+             or node.__class__.__name__ not in self.optional):
+            raise NotImplementedError(
+                '%s departing unknown node type: %s'
+                % (self.__class__, node.__class__.__name__))
 
 
 class SparseNodeVisitor(NodeVisitor):
