@@ -1,7 +1,7 @@
 # Author: David Goodger
 # Contact: goodger@users.sourceforge.net
-# Revision: $Revision: 1.21 $
-# Date: $Date: 2004/05/04 00:38:15 $
+# Revision: $Revision: 1.22 $
+# Date: $Date: 2004/09/30 13:47:58 $
 # Copyright: This module has been placed in the public domain.
 
 """
@@ -700,7 +700,22 @@ class Substitutions(Transform):
                     msg.add_backref(prbid)
                     ref.parent.replace(ref, prb)
                 else:
-                    ref.parent.replace(ref, defs[key].get_children())
+                    subdef = defs[key]
+                    parent = ref.parent
+                    index = parent.index(ref)
+                    if  (subdef.attributes.has_key('ltrim')
+                         or subdef.attributes.has_key('trim')):
+                        if index > 0 and isinstance(parent[index - 1],
+                                                    nodes.Text):
+                            parent.replace(parent[index - 1],
+                                           parent[index - 1].rstrip())
+                    if  (subdef.attributes.has_key('rtrim')
+                         or subdef.attributes.has_key('trim')):
+                        if  (len(parent) > index + 1
+                             and isinstance(parent[index + 1], nodes.Text)):
+                            parent.replace(parent[index + 1],
+                                           parent[index + 1].lstrip())
+                    parent.replace(ref, subdef.get_children())
         self.document.substitution_refs = None  # release replaced references
 
 
@@ -767,7 +782,7 @@ class TargetNotes(Transform):
             self.document.note_footnote_ref(refnode)
             index = ref.parent.index(ref) + 1
             reflist = [refnode]
-            if not self.document.settings.trim_footnote_reference_space:
+            if not utils.get_trim_footnote_ref_space(self.document.settings):
                 reflist.insert(0, nodes.Text(' '))
             ref.parent.insert(index, reflist)
         return footnote
