@@ -1,7 +1,7 @@
 # Author: David Goodger
 # Contact: goodger@users.sourceforge.net
-# Revision: $Revision: 1.31 $
-# Date: $Date: 2004/04/27 19:51:07 $
+# Revision: $Revision: 1.40 $
+# Date: $Date: 2004/09/30 13:47:58 $
 # Copyright: This module has been placed in the public domain.
 
 """
@@ -25,6 +25,9 @@ class SystemMessage(ApplicationError):
         self.level = level
 
 
+class SystemMessagePropagation(ApplicationError): pass
+
+
 class Reporter:
 
     """
@@ -52,7 +55,7 @@ class Reporter:
     is compared to the thresholds stored in the category, and a warning or
     error is generated as appropriate.  Debug messages are produced iff the
     stored debug switch is on.  Message output is sent to the stored warning
-    stream.
+    stream if not set to ''.
 
     The default category is '' (empty string).  By convention, Writers should
     retrieve reporting conditions from the 'writer' category (which, unless
@@ -89,18 +92,24 @@ class Reporter:
               exceptions will be raised, halting execution.
             - `debug`: Show debug (level=0) system messages?
             - `stream`: Where warning output is sent.  Can be file-like (has a
-              ``.write`` method), a string (file name, opened for writing), or
+              ``.write`` method), a string (file name, opened for writing),
+              '' (empty string, for discarding all stream messages) or
               `None` (implies `sys.stderr`; default).
             - `encoding`: The encoding for stderr output.
             - `error_handler`: The error handler for stderr output encoding.
         """
         self.source = source
         """The path to or description of the source data."""
-        
+
         if stream is None:
             stream = sys.stderr
         elif type(stream) in (StringType, UnicodeType):
-            raise NotImplementedError('This should open a file for writing.')
+            # Leave stream untouched if it's ''.
+            if stream != '':
+                if type(stream) == StringType:
+                    stream = open(stream, 'w')
+                elif type(stream) == UnicodeType:
+                    stream = open(stream.encode(), 'w')
 
         self.encoding = encoding
         """The character encoding for the stderr output."""
@@ -175,7 +184,7 @@ class Reporter:
                                    type=self.levels[level],
                                    *children, **attributes)
         debug, report_level, halt_level, stream = self[category].astuple()
-        if level >= report_level or debug and level == 0:
+        if (level >= report_level or debug and level == 0) and stream:
             msgtext = msg.astext().encode(self.encoding, self.error_handler)
             if category:
                 print >>stream, msgtext, '[%s]' % category
@@ -265,6 +274,8 @@ def extract_extension_options(field_list, options_spec):
         - `KeyError` for unknown option names.
         - `ValueError` for invalid option values (raised by the conversion
            function).
+        - `TypeError` for invalid option value types (raised by conversion
+           function).
         - `DuplicateOptionError` for duplicate options.
         - `BadOptionError` for invalid fields.
         - `BadOptionDataError` for invalid option data (missing name,
@@ -321,6 +332,8 @@ def assemble_option_dict(option_list, options_spec):
         - `DuplicateOptionError` for duplicate options.
         - `ValueError` for invalid option values (raised by conversion
            function).
+        - `TypeError` for invalid option value types (raised by conversion
+           function).
     """
     options = {}
     for name, value in option_list:
@@ -437,6 +450,34 @@ def relative_path(source, target):
     parts = ['..'] * (len(source_parts) - 1) + target_parts
     return '/'.join(parts)
 
+def get_stylesheet_reference(settings, relative_to=None):
+    """
+    Retrieve a stylesheet reference from the settings object.
+    """
+    if settings.stylesheet_path:
+        assert not settings.stylesheet, \
+               'stylesheet and stylesheet_path are mutually exclusive.'
+        if relative_to == None:
+            relative_to = settings._destination
+        return relative_path(relative_to, settings.stylesheet_path)
+    else:
+        return settings.stylesheet
+
+def get_trim_footnote_ref_space(settings):
+    """
+    Return whether or not to trim footnote space.
+
+    If trim_footnote_reference_space is not None, return it.
+
+    If trim_footnote_reference_space is None, return False unless the
+    footnote reference style is 'superscript'.
+    """
+    if settings.trim_footnote_reference_space is None:
+        return hasattr(settings, 'footnote_references') and \
+               settings.footnote_references == 'superscript'
+    else:
+        return settings.trim_footnote_reference_space
+
 def get_source_line(node):
     """
     Return the "source" and "line" attributes from the `node` given or from
@@ -447,3 +488,92 @@ def get_source_line(node):
             return node.source, node.line
         node = node.parent
     return None, None
+
+def escape2null(text):
+    """Return a string with escape-backslashes converted to nulls."""
+    parts = []
+    start = 0
+    while 1:
+        found = text.find('\\', start)
+        if found == -1:
+            parts.append(text[start:])
+            return ''.join(parts)
+        parts.append(text[start:found])
+        parts.append('\x00' + text[found+1:found+2])
+        start = found + 2               # skip character after escape
+
+def unescape(text, restore_backslashes=0):
+    """
+    Return a string with nulls removed or restored to backslashes.
+    Backslash-escaped spaces are also removed.
+    """
+    if restore_backslashes:
+        return text.replace('\x00', '\\')
+    else:
+        for sep in ['\x00 ', '\x00\n', '\x00']:
+            text = ''.join(text.split(sep))
+        return text
+
+
+class DependencyList:
+
+    """
+    List of dependencies, with file recording support.
+
+    Note that the output file is not automatically closed.  You have
+    to explicitly call the close() method.
+    """
+
+    def __init__(self, output_file=None, dependencies=[]):
+        """
+        Initialize the dependency list, automatically setting the
+        output file to `output_file` (see `set_output()`) and adding
+        all supplied dependencies.
+        """
+        self.set_output(output_file)
+        for i in dependencies:
+            self.add(i)
+
+    def set_output(self, output_file):
+        """
+        Set the output file and clear the list of already added
+        dependencies.
+
+        `output_file` must be a string.  The specified file is
+        immediately overwritten.
+
+        If output_file is '-', the output will be written to stdout.
+        If it is None, no file output is done when calling add().
+        """
+        self.list = []
+        if output_file == '-':
+            self.file = sys.stdout
+        elif output_file:
+            self.file = open(output_file, 'w')
+        else:
+            self.file = None
+
+    def add(self, filename):
+        """
+        If the dependency `filename` has not already been added,
+        append it to self.list and print it to self.file if self.file
+        is not None.
+        """
+        if not filename in self.list:
+            self.list.append(filename)
+            if self.file is not None:
+                print >>self.file, filename
+
+    def close(self):
+        """
+        Close the output file.
+        """
+        self.file.close()
+        self.file = None
+
+    def __repr__(self):
+        if self.file:
+            output_file = self.file.name
+        else:
+            output_file = None
+        return '%s(%r, %s)' % (self.__class__.__name__, output_file, self.list)
