@@ -1,7 +1,7 @@
 # Author: David Goodger
 # Contact: goodger@users.sourceforge.net
-# Revision: $Revision: 1.109 $
-# Date: $Date: 2004/05/08 22:47:28 $
+# Revision: $Revision: 1.142 $
+# Date: $Date: 2004/11/26 16:49:56 $
 # Copyright: This module has been placed in the public domain.
 
 """
@@ -39,14 +39,15 @@ class Writer(writers.Writer):
         'HTML-Specific Options',
         None,
         (('Specify a stylesheet URL, used verbatim.  Default is '
-          '"default.css".  Overridden by --stylesheet-path.',
+          '"default.css".  Overrides --stylesheet-path.',
           ['--stylesheet'],
-          {'default': 'default.css', 'metavar': '<URL>'}),
+          {'default': 'default.css', 'metavar': '<URL>',
+           'overrides': 'stylesheet_path'}),
          ('Specify a stylesheet file, relative to the current working '
           'directory.  The path is adjusted relative to the output HTML '
           'file.  Overrides --stylesheet.',
           ['--stylesheet-path'],
-          {'metavar': '<file>'}),
+          {'metavar': '<file>', 'overrides': 'stylesheet'}),
          ('Link to the stylesheet in the output HTML file.  This is the '
           'default.',
           ['--link-stylesheet'],
@@ -54,9 +55,7 @@ class Writer(writers.Writer):
            'validator': frontend.validate_boolean}),
          ('Embed the stylesheet in the output HTML file.  The stylesheet '
           'file must be accessible during processing (--stylesheet-path is '
-          'recommended).  The stylesheet is embedded inside a comment, so it '
-          'must not contain the text "--" (two hyphens).  Default: link the '
-          'stylesheet, do not embed it.',
+          'recommended).  Default: link the stylesheet, do not embed it.',
           ['--embed-stylesheet'],
           {'action': 'store_true', 'validator': frontend.validate_boolean}),
          ('Specify the initial header level.  Default is 1 for "<h1>".  '
@@ -65,10 +64,11 @@ class Writer(writers.Writer):
           {'choices': '1 2 3 4 5 6'.split(), 'default': '1',
            'metavar': '<level>'}),
          ('Format for footnote references: one of "superscript" or '
-          '"brackets".  Default is "superscript".',
+          '"brackets".  Default is "brackets".',
           ['--footnote-references'],
-          {'choices': ['superscript', 'brackets'], 'default': 'superscript',
-           'metavar': '<format>'}),
+          {'choices': ['superscript', 'brackets'], 'default': 'brackets',
+           'metavar': '<format>',
+           'overrides': 'trim_footnote_reference_space'}),
          ('Format for block quote attributions: one of "dash" (em-dash '
           'prefix), "parentheses"/"parens", or "none".  Default is "dash".',
           ['--attribution'],
@@ -170,7 +170,7 @@ class HTMLTranslator(nodes.NodeVisitor):
     generator = ('<meta name="generator" content="Docutils %s: '
                  'http://docutils.sourceforge.net/" />\n')
     stylesheet_link = '<link rel="stylesheet" href="%s" type="text/css" />\n'
-    embedded_stylesheet = '<style type="text/css"><!--\n\n%s\n--></style>\n'
+    embedded_stylesheet = '<style type="text/css">\n\n%s\n</style>\n'
     named_tags = {'a': 1, 'applet': 1, 'form': 1, 'frame': 1, 'iframe': 1,
                   'img': 1, 'map': 1}
     words_and_spaces = re.compile(r'\S+| +|\n')
@@ -191,12 +191,13 @@ class HTMLTranslator(nodes.NodeVisitor):
                                     % settings.output_encoding)
         self.head = []
         if settings.embed_stylesheet:
-            stylesheet = self.get_stylesheet_reference(
+            stylesheet = utils.get_stylesheet_reference(settings,
                 os.path.join(os.getcwd(), 'dummy'))
+            settings.record_dependencies.add(stylesheet)
             stylesheet_text = open(stylesheet).read()
             self.stylesheet = [self.embedded_stylesheet % stylesheet_text]
         else:
-            stylesheet = self.get_stylesheet_reference()
+            stylesheet = utils.get_stylesheet_reference(settings)
             if stylesheet:
                 self.stylesheet = [self.stylesheet_link % stylesheet]
             else:
@@ -226,15 +227,6 @@ class HTMLTranslator(nodes.NodeVisitor):
         self.footer = []
         self.in_document_title = 0
 
-    def get_stylesheet_reference(self, relative_to=None):
-        settings = self.settings
-        if settings.stylesheet_path:
-            if relative_to == None:
-                relative_to = settings._destination
-            return utils.relative_path(relative_to, settings.stylesheet_path)
-        else:
-            return settings.stylesheet
-
     def astext(self):
         return ''.join(self.head_prefix + self.head
                        + self.stylesheet + self.body_prefix
@@ -249,6 +241,8 @@ class HTMLTranslator(nodes.NodeVisitor):
         text = text.replace('"', "&quot;")
         text = text.replace(">", "&gt;")
         text = text.replace("@", "&#64;") # may thwart some address harvesters
+        # Replace the non-breaking space character with the HTML entity:
+        text = text.replace(u'\u00a0', "&nbsp;")
         return text
 
     def attval(self, text,
@@ -278,12 +272,10 @@ class HTMLTranslator(nodes.NodeVisitor):
         attlist.sort()
         parts = [tagname]
         for name, value in attlist:
-            if value is None:           # boolean attribute
-                # According to the HTML spec, ``<element boolean>`` is good,
-                # ``<element boolean="boolean">`` is bad.
-                # (But the XHTML (XML) spec says the opposite.  <sigh>)
-                parts.append(name.lower())
-            elif isinstance(value, ListType):
+            # value=None was used for boolean attributes without
+            # value, but this isn't supported by XHTML.
+            assert value is not None
+            if isinstance(value, ListType):
                 values = [unicode(v) for v in value]
                 parts.append('%s="%s"' % (name.lower(),
                                           self.attval(' '.join(values))))
@@ -299,6 +291,11 @@ class HTMLTranslator(nodes.NodeVisitor):
         """Construct and return an XML-compatible empty tag."""
         return self.starttag(node, tagname, suffix, infix=' /', **attributes)
 
+    def set_first_last(self, node):
+        if len(node):
+            node[0].set_class('first')
+            node[-1].set_class('last')
+
     def visit_Text(self, node):
         self.body.append(self.encode(node.astext()))
 
@@ -331,8 +328,8 @@ class HTMLTranslator(nodes.NodeVisitor):
         self.body.append(self.starttag(node, 'div',
                                         CLASS=(name or 'admonition')))
         if name:
-            self.body.append('<p class="admonition-title first">'
-                             + self.language.labels[name] + '</p>\n')
+            node.insert(0, nodes.title(name, self.language.labels[name]))
+        self.set_first_last(node)
 
     def depart_admonition(self, node=None):
         self.body.append('</div>\n')
@@ -415,10 +412,10 @@ class HTMLTranslator(nodes.NodeVisitor):
         self.depart_admonition()
 
     def visit_citation(self, node):
-        self.body.append(self.starttag(node, 'table', CLASS='citation',
+        self.body.append(self.starttag(node, 'table',
+                                       CLASS='docutils citation',
                                        frame="void", rules="none"))
         self.body.append('<colgroup><col class="label" /><col /></colgroup>\n'
-                         '<col />\n'
                          '<tbody valign="top">\n'
                          '<tr>')
         self.footnote_backrefs(node)
@@ -433,8 +430,9 @@ class HTMLTranslator(nodes.NodeVisitor):
             href = '#' + node['refid']
         elif node.has_key('refname'):
             href = '#' + self.document.nameids[node['refname']]
-        self.body.append(self.starttag(node, 'a', '[', href=href,
-                                       CLASS='citation-reference'))
+        self.body.append(self.starttag(node, 'a', '[',
+                                       CLASS='citation-reference',
+                                       **(href and {'href': href} or {})))
 
     def depart_citation_reference(self, node):
         self.body.append(']</a>')
@@ -469,6 +467,17 @@ class HTMLTranslator(nodes.NodeVisitor):
         # Content already processed:
         raise nodes.SkipNode
 
+    def visit_compound(self, node):
+        self.body.append(self.starttag(node, 'div', CLASS='compound'))
+        if len(node) > 1:
+            node[0].set_class('compound-first')
+            node[-1].set_class('compound-last')
+            for child in node[1:-1]:
+                child.set_class('compound-middle')
+
+    def depart_compound(self, node):
+        self.body.append('</div>\n')
+
     def visit_contact(self, node):
         self.visit_docinfo_item(node, 'contact', meta=None)
 
@@ -502,15 +511,13 @@ class HTMLTranslator(nodes.NodeVisitor):
     def visit_definition(self, node):
         self.body.append('</dt>\n')
         self.body.append(self.starttag(node, 'dd', ''))
-        if len(node):
-            node[0].set_class('first')
-            node[-1].set_class('last')
+        self.set_first_last(node)
 
     def depart_definition(self, node):
         self.body.append('</dd>\n')
 
     def visit_definition_list(self, node):
-        self.body.append(self.starttag(node, 'dl'))
+        self.body.append(self.starttag(node, 'dl', CLASS='docutils'))
 
     def depart_definition_list(self, node):
         self.body.append('</dl>\n')
@@ -523,16 +530,15 @@ class HTMLTranslator(nodes.NodeVisitor):
 
     def visit_description(self, node):
         self.body.append(self.starttag(node, 'td', ''))
-        if len(node):
-            node[0].set_class('first')
-            node[-1].set_class('last')
+        self.set_first_last(node)
 
     def depart_description(self, node):
         self.body.append('</td>')
 
     def visit_docinfo(self, node):
         self.context.append(len(self.body))
-        self.body.append(self.starttag(node, 'table', CLASS='docinfo',
+        self.body.append(self.starttag(node, 'table',
+                                       CLASS='docinfo',
                                        frame="void", rules="none"))
         self.body.append('<col class="docinfo-name" />\n'
                          '<col class="docinfo-content" />\n'
@@ -577,8 +583,8 @@ class HTMLTranslator(nodes.NodeVisitor):
 
     def depart_document(self, node):
         self.fragment.extend(self.body)
-        self.body.insert(0, self.starttag(node, 'div', CLASS='document'))
-        self.body.append('</div>\n')
+        self.body_prefix.append(self.starttag(node, 'div', CLASS='document'))
+        self.body_suffix.insert(0, '</div>\n')
 
     def visit_emphasis(self, node):
         self.body.append('<em>')
@@ -600,9 +606,7 @@ class HTMLTranslator(nodes.NodeVisitor):
         self.context.append('</%s>\n' % tagname.lower())
         if len(node) == 0:              # empty cell
             self.body.append('&nbsp;')
-        else:
-            node[0].set_class('first')
-            node[-1].set_class('last')
+        self.set_first_last(node)
 
     def depart_entry(self, node):
         self.body.append(self.context.pop())
@@ -649,16 +653,15 @@ class HTMLTranslator(nodes.NodeVisitor):
 
     def visit_field_body(self, node):
         self.body.append(self.starttag(node, 'td', '', CLASS='field-body'))
-        if len(node):
-            node[0].set_class('first')
-            node[-1].set_class('last')
+        self.set_first_last(node)
 
     def depart_field_body(self, node):
         self.body.append('</td>\n')
 
     def visit_field_list(self, node):
         self.body.append(self.starttag(node, 'table', frame='void',
-                                       rules='none', CLASS='field-list'))
+                                       rules='none',
+                                       CLASS='docutils field-list'))
         self.body.append('<col class="field-name" />\n'
                          '<col class="field-body" />\n'
                          '<tbody valign="top">\n')
@@ -697,7 +700,7 @@ class HTMLTranslator(nodes.NodeVisitor):
 
     def depart_footer(self, node):
         start = self.context.pop()
-        footer = (['<hr class="footer" />\n',
+        footer = (['<hr class="docutils footer" />\n',
                    self.starttag(node, 'div', CLASS='footer')]
                   + self.body[start:] + ['</div>\n'])
         self.footer.extend(footer)
@@ -705,7 +708,8 @@ class HTMLTranslator(nodes.NodeVisitor):
         del self.body[start:]
 
     def visit_footnote(self, node):
-        self.body.append(self.starttag(node, 'table', CLASS='footnote',
+        self.body.append(self.starttag(node, 'table',
+                                       CLASS='docutils footnote',
                                        frame="void", rules="none"))
         self.body.append('<colgroup><col class="label" /><col /></colgroup>\n'
                          '<tbody valign="top">\n'
@@ -713,6 +717,7 @@ class HTMLTranslator(nodes.NodeVisitor):
         self.footnote_backrefs(node)
 
     def footnote_backrefs(self, node):
+        backlinks = []
         if self.settings.footnote_backlinks and node.hasattr('backrefs'):
             backrefs = node['backrefs']
             if len(backrefs) == 1:
@@ -721,7 +726,6 @@ class HTMLTranslator(nodes.NodeVisitor):
                                     'name="%s">' % (backrefs[0], node['id']))
             else:
                 i = 1
-                backlinks = []
                 for backref in backrefs:
                     backlinks.append('<a class="fn-backref" href="#%s">%s</a>'
                                      % (backref, i))
@@ -731,6 +735,13 @@ class HTMLTranslator(nodes.NodeVisitor):
         else:
             self.context.append('')
             self.context.append('<a name="%s">' % node['id'])
+        # If the node does not only consist of a label.
+        if len(node) > 1:
+            # If there are preceding backlinks, we do not set class
+            # 'first', because we need to retain the top-margin.
+            if not backlinks:
+                node[1].set_class('first')
+            node[-1].set_class('last')
 
     def depart_footnote(self, node):
         self.body.append('</td></tr>\n'
@@ -752,8 +763,9 @@ class HTMLTranslator(nodes.NodeVisitor):
         else:                           # shouldn't happen
             suffix = '???'
             self.content.append('???')
-        self.body.append(self.starttag(node, 'a', suffix, href=href,
-                                       CLASS='footnote-reference'))
+        self.body.append(self.starttag(node, 'a', suffix,
+                                       CLASS='footnote-reference',
+                                       **(href and {'href': href} or {})))
 
     def depart_footnote_reference(self, node):
         self.body.append(self.context.pop() + '</a>')
@@ -771,7 +783,7 @@ class HTMLTranslator(nodes.NodeVisitor):
         start = self.context.pop()
         header = [self.starttag(node, 'div', CLASS='header')]
         header.extend(self.body[start:])
-        header.append('<hr />\n</div>\n')
+        header.append('<hr class="docutils header"/>\n</div>\n')
         self.body_prefix.extend(header)
         self.header = header
         del self.body[start:]
@@ -814,14 +826,20 @@ class HTMLTranslator(nodes.NodeVisitor):
         if isinstance(node.parent, nodes.TextElement):
             self.context.append('')
         else:
-            if atts.has_key('align'):
-                self.body.append('<p align="%s">' %
-                                 (self.attval(atts['align'],)))
-            else:
-                self.body.append('<p>')
-            self.context.append('</p>\n')
+            div_atts = self.image_div_atts(node)
+            self.body.append(self.starttag({}, 'div', '', **div_atts))
+            self.context.append('</div>\n')
         self.body.append(self.emptytag(node, 'img', '', **atts))
 
+    def image_div_atts(self, image_node):
+        div_atts = {'class': 'image'}
+        if image_node.attributes.has_key('class'):
+            div_atts['class'] += ' ' + image_node.attributes['class']
+        if image_node.attributes.has_key('align'):
+            div_atts['align'] = self.attval(image_node.attributes['align'])
+            div_atts['class'] += ' align-%s' % div_atts['align']
+        return div_atts
+
     def depart_image(self, node):
         self.body.append(self.context.pop())
 
@@ -850,11 +868,19 @@ class HTMLTranslator(nodes.NodeVisitor):
     def depart_legend(self, node):
         self.body.append('</div>\n')
 
+    def visit_line(self, node):
+        self.body.append(self.starttag(node, 'div', suffix='', CLASS='line'))
+        if not len(node):
+            self.body.append('<br />')
+
+    def depart_line(self, node):
+        self.body.append('</div>\n')
+
     def visit_line_block(self, node):
-        self.body.append(self.starttag(node, 'pre', CLASS='line-block'))
+        self.body.append(self.starttag(node, 'div', CLASS='line-block'))
 
     def depart_line_block(self, node):
-        self.body.append('\n</pre>\n')
+        self.body.append('</div>\n')
 
     def visit_list_item(self, node):
         self.body.append(self.starttag(node, 'li', ''))
@@ -866,7 +892,7 @@ class HTMLTranslator(nodes.NodeVisitor):
 
     def visit_literal(self, node):
         """Process text to prevent tokens from wrapping."""
-        self.body.append(self.starttag(node, 'tt', '', CLASS='literal'))
+        self.body.append(self.starttag(node, 'tt', '', CLASS='docutils literal'))
         text = node.astext()
         for token in self.words_and_spaces.findall(text):
             if token.strip():
@@ -938,7 +964,7 @@ class HTMLTranslator(nodes.NodeVisitor):
 
     def visit_option_list(self, node):
         self.body.append(
-              self.starttag(node, 'table', CLASS='option-list',
+              self.starttag(node, 'table', CLASS='docutils option-list',
                             frame="void", rules="none"))
         self.body.append('<col class="option" />\n'
                          '<col class="description" />\n'
@@ -965,12 +991,25 @@ class HTMLTranslator(nodes.NodeVisitor):
     def depart_organization(self, node):
         self.depart_docinfo_item()
 
+    def should_be_compact_paragraph(self, node):
+        """
+        Determine if the <p> tags around paragraph ``node`` can be omitted.
+        """
+        if (isinstance(node.parent, nodes.document) or
+            isinstance(node.parent, nodes.compound)):
+            # Never compact paragraphs in document or compound.
+            return 0
+        if ((node.attributes in ({}, {'class': 'first'}, {'class': 'last'},
+                                 {'class': 'first last'})) and
+            (self.compact_simple or
+             self.compact_p and (len(node.parent) == 1 or
+                                 len(node.parent) == 2 and
+                                 isinstance(node.parent[0], nodes.label)))):
+            return 1
+        return 0
+
     def visit_paragraph(self, node):
-        # Omit <p> tags if this is an only child and optimizable.
-        if (self.compact_simple or
-            self.compact_p and (len(node.parent) == 1 or
-                                len(node.parent) == 2 and
-                                isinstance(node.parent[0], nodes.label))):
+        if self.should_be_compact_paragraph(node):
             self.context.append('')
         else:
             self.body.append(self.starttag(node, 'p', ''))
@@ -993,25 +1032,35 @@ class HTMLTranslator(nodes.NodeVisitor):
         self.body.append(self.context.pop())
 
     def visit_raw(self, node):
-        if node.get('format') == 'html':
+        if 'html' in node.get('format', '').split():
+            add_class = node.attributes.get('class') is not None
+            t = isinstance(node.parent, nodes.TextElement) and 'span' or 'div'
+            if add_class:
+                self.body.append(self.starttag(node, t, suffix=''))
             self.body.append(node.astext())
+            if add_class:
+                self.body.append('</%s>' % t)
         # Keep non-HTML raw text out of output:
         raise nodes.SkipNode
 
     def visit_reference(self, node):
         if isinstance(node.parent, nodes.TextElement):
             self.context.append('')
-        else:
-            self.body.append('<p>')
-            self.context.append('</p>\n')
+        else:                           # contains an image
+            assert len(node) == 1 and isinstance(node[0], nodes.image)
+            div_atts = self.image_div_atts(node[0])
+            div_atts['class'] += ' image-reference'
+            self.body.append(self.starttag({}, 'div', '', **div_atts))
+            self.context.append('</div>\n')
+        href = ''
         if node.has_key('refuri'):
             href = node['refuri']
         elif node.has_key('refid'):
             href = '#' + node['refid']
         elif node.has_key('refname'):
             href = '#' + self.document.nameids[node['refname']]
-        self.body.append(self.starttag(node, 'a', '', href=href,
-                                       CLASS='reference'))
+        self.body.append(self.starttag(node, 'a', '', CLASS='reference',
+                                       **(href and {'href': href} or {})))
 
     def depart_reference(self, node):
         self.body.append('</a>')
@@ -1045,6 +1094,7 @@ class HTMLTranslator(nodes.NodeVisitor):
 
     def visit_sidebar(self, node):
         self.body.append(self.starttag(node, 'div', CLASS='sidebar'))
+        self.set_first_last(node)
         self.in_sidebar = 1
 
     def depart_sidebar(self, node):
@@ -1132,7 +1182,8 @@ class HTMLTranslator(nodes.NodeVisitor):
             a_end = '</a>'
         else:
             a_start = a_end = ''
-        self.body.append('System Message: %s%s/%s%s (<tt>%s</tt>%s)%s</p>\n'
+        self.body.append('System Message: %s%s/%s%s '
+                         '(<tt class="docutils">%s</tt>%s)%s</p>\n'
                          % (a_start, node['type'], node['level'], a_end,
                             self.encode(node['source']), line, backref_text))
 
@@ -1141,9 +1192,7 @@ class HTMLTranslator(nodes.NodeVisitor):
 
     def visit_table(self, node):
         self.body.append(
-              # "border=None" is a boolean attribute;
-              # it means "standard border", not "no border":
-              self.starttag(node, 'table', CLASS="table", border=None))
+            self.starttag(node, 'table', CLASS='docutils', border="1"))
 
     def depart_table(self, node):
         self.body.append('</table>\n')
@@ -1212,11 +1261,11 @@ class HTMLTranslator(nodes.NodeVisitor):
             check_id = 1
         elif isinstance(node.parent, nodes.sidebar):
             self.body.append(
-                  self.starttag(node, 'p', '', CLASS='sidebar-title first'))
+                  self.starttag(node, 'p', '', CLASS='sidebar-title'))
             check_id = 1
-        elif isinstance(node.parent, nodes.admonition):
+        elif isinstance(node.parent, nodes.Admonition):
             self.body.append(
-                  self.starttag(node, 'p', '', CLASS='admonition-title first'))
+                  self.starttag(node, 'p', '', CLASS='admonition-title'))
             check_id = 1
         elif isinstance(node.parent, nodes.table):
             self.body.append(
@@ -1273,7 +1322,7 @@ class HTMLTranslator(nodes.NodeVisitor):
         self.topic_class = ''
 
     def visit_transition(self, node):
-        self.body.append(self.emptytag(node, 'hr'))
+        self.body.append(self.emptytag(node, 'hr', CLASS='docutils'))
 
     def depart_transition(self, node):
         pass
@@ -1298,7 +1347,7 @@ class HTMLTranslator(nodes.NodeVisitor):
 class SimpleListChecker(nodes.GenericNodeVisitor):
 
     """
-    Raise `nodes.SkipNode` if non-simple list item is encountered.
+    Raise `nodes.NodeFound` if non-simple list item is encountered.
 
     Here "simple" means a list item containing nothing other than a single
     paragraph, a simple list, or a paragraph followed by a simple list.
@@ -1332,7 +1381,7 @@ class SimpleListChecker(nodes.GenericNodeVisitor):
 
     def invisible_visit(self, node):
         """Invisible nodes should be ignored."""
-        pass
+        raise nodes.SkipNode
 
     visit_comment = invisible_visit
     visit_substitution_definition = invisible_visit
