@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-# $Id: install.py,v 1.2 2003/06/09 21:26:29 goodger Exp $
+# $Id: install.py,v 1.3 2004/07/13 15:57:13 goodger Exp $
 # Copyright: This file has been placed in the public domain.
 
 """
@@ -8,13 +8,19 @@ command::
 
     python setup.py install
 
-However, the shortcut lacks error checking!
+However, the shortcut lacks error checking and command-line option
+processing.  If you need any kind of customization or help, please use
+one of::
+
+    python setup.py install --help
+    python setup.py --help
 """
 
 from distutils import core
 from setup import do_setup
 
 if __name__ == '__main__' :
+    print __doc__
     core._setup_stop_after = 'config'
     dist = do_setup()
     dist.commands = ['install']
