@@ -1,7 +1,7 @@
 # Authors:  David Goodger; Garth Kidd
 # Contact: goodger@users.sourceforge.net
-# Revision: $Revision: 1.38 $
-# Date: $Date: 2004/05/09 02:14:33 $
+# Revision: $Revision: 1.54 $
+# Date: $Date: 2004/10/30 15:54:45 $
 # Copyright: This module has been placed in the public domain.
 
 """
@@ -17,18 +17,19 @@ Exports the following:
     - `tableparser` is 'docutils.parsers.rst.tableparser'
 
 :Classes:
-    - `CustomTestSuite`
     - `CustomTestCase`
-    - `TransformTestSuite`
+    - `CustomTestSuite`
     - `TransformTestCase`
-    - `ParserTestSuite`
+    - `TransformTestSuite`
     - `ParserTestCase`
-    - `PEPParserTestSuite`
+    - `ParserTestSuite`
+    - `ParserTransformTestCase`
     - `PEPParserTestCase`
-    - `GridTableParserTestSuite`
+    - `PEPParserTestSuite`
     - `GridTableParserTestCase`
-    - `SimpleTableParserTestSuite`
+    - `GridTableParserTestSuite`
     - `SimpleTableParserTestCase`
+    - `SimpleTableParserTestSuite`
     - `WriterPublishTestCase`
     - `LatexWriterPublishTestCase`
     - `PseudoXMLWriterPublishTestCase`
@@ -42,10 +43,10 @@ __docformat__ = 'reStructuredText'
 import sys
 import os
 import unittest
-import difflib
+import docutils_difflib
 import inspect
 from pprint import pformat
-from types import UnicodeType
+from types import UnicodeType, StringType
 import package_unittest
 import docutils
 import docutils.core
@@ -56,9 +57,11 @@ from docutils.parsers.rst import states, tableparser, roles, languages
 from docutils.readers import standalone, pep
 from docutils.statemachine import StringList, string2lines
 
-if sys.hexversion >= 0x02020000:    # Python 2.2
+try:
     from docutils.readers.python import moduleparser
-else:
+    from tokenize import generate_tokens
+    del generate_tokens
+except ImportError:      # moduleparser depends on modules added in Python 2.2
     moduleparser = None
 
 try:
@@ -71,6 +74,9 @@ except:
 StringList.__repr__ = StringList.__str__
 
 
+testroot = os.path.abspath(os.path.dirname(__file__) or os.curdir)
+
+
 class DevNull:
 
     """Output sink."""
@@ -78,12 +84,102 @@ class DevNull:
     def write(self, string):
         pass
 
+    def close(self):
+        pass
+
+
+class CustomTestCase(unittest.TestCase):
+    
+    """
+    Helper class, providing extended functionality over unittest.TestCase.
+
+    This isn't specific to Docutils but of general use when dealing
+    with large amounts of text.  The methods failUnlessEqual and
+    failIfEqual have been overwritten to provide better support for
+    multi-line strings.  Furthermore, see the compare_output method
+    and the parameter list of __init__.
+    """
+
+    compare = docutils_difflib.Differ().compare
+    """Comparison method shared by all subclasses."""
+
+    def __init__(self, method_name, input, expected, id, run_in_debugger=0):
+        """
+        Initialise the CustomTestCase.
+
+        Arguments:
+
+        method_name -- name of test method to run.
+        input -- input to the parser.
+        expected -- expected output from the parser.
+        id -- unique test identifier, used by the test framework.
+        run_in_debugger -- if true, run this test under the pdb debugger.
+        """
+        self.id = id
+        self.input = input
+        self.expected = expected
+        self.run_in_debugger = run_in_debugger
+        
+        # Ring your mother.
+        unittest.TestCase.__init__(self, method_name)
+
+    def __str__(self):
+        """
+        Return string conversion. Overridden to give test id, in addition to
+        method name.
+        """
+        return '%s; %s' % (self.id, unittest.TestCase.__str__(self))
+
+    def __repr__(self):
+        return "<%s %s>" % (self.id, unittest.TestCase.__repr__(self))
+
+    def compare_output(self, input, output, expected):
+        """`input`, `output`, and `expected` should all be strings."""
+        if type(input) == UnicodeType:
+            input = input.encode('raw_unicode_escape')
+        if type(output) == UnicodeType:
+            output = output.encode('raw_unicode_escape')
+        if type(expected) == UnicodeType:
+            expected = expected.encode('raw_unicode_escape')
+        try:
+            self.assertEquals(output, expected)
+        except AssertionError:
+            print >>sys.stderr, '\n%s\ninput:' % (self,)
+            print >>sys.stderr, input
+            print >>sys.stderr, '-: expected\n+: output'
+            print >>sys.stderr, ''.join(self.compare(expected.splitlines(1),
+                                                     output.splitlines(1)))
+            raise
+
+    def failUnlessEqual(self, first, second, msg=None):
+        """Fail if the two objects are unequal as determined by the '=='
+           operator.
+        """
+        if not first == second:
+            raise self.failureException, \
+                  (msg or '%s != %s' % _format_str(first, second))
+
+    def failIfEqual(self, first, second, msg=None):
+        """Fail if the two objects are equal as determined by the '=='
+           operator.
+        """
+        if first == second:
+            raise self.failureException, \
+                  (msg or '%s == %s' % _format_str(first, second))
+
+    # Synonyms for assertion methods
+
+    assertEqual = assertEquals = failUnlessEqual
+
+    assertNotEqual = assertNotEquals = failIfEqual
+
 
 class CustomTestSuite(unittest.TestSuite):
 
     """
-    A collection of custom TestCases.
+    A collection of CustomTestCases.
 
+    Provides test suite ID generation and a method for adding test cases.
     """
 
     id = ''
@@ -125,21 +221,19 @@ class CustomTestSuite(unittest.TestSuite):
             self.id = id
 
     def addTestCase(self, test_case_class, method_name, input, expected,
-                    id=None, run_in_debugger=0, short_description=None,
-                    **kwargs):
+                    id=None, run_in_debugger=0, **kwargs):
         """
-        Create a custom TestCase in the CustomTestSuite.
+        Create a CustomTestCase in the CustomTestSuite.
         Also return it, just in case.
 
         Arguments:
 
-        test_case_class --
-        method_name --
+        test_case_class -- the CustomTestCase to add
+        method_name -- a string; CustomTestCase.method_name is the test
         input -- input to the parser.
         expected -- expected output from the parser.
         id -- unique test identifier, used by the test framework.
         run_in_debugger -- if true, run this test under the pdb debugger.
-        short_description -- override to default test description.
         """
         if id is None:                  # generate id if required
             id = self.next_test_case_id
@@ -148,9 +242,7 @@ class CustomTestSuite(unittest.TestSuite):
         tcid = '%s: %s' % (self.id, id)
         # generate and add test case
         tc = test_case_class(method_name, input, expected, tcid,
-                             run_in_debugger=run_in_debugger,
-                             short_description=short_description,
-                             **kwargs)
+                             run_in_debugger=run_in_debugger, **kwargs)
         self.addTest(tc)
         return tc
 
@@ -158,109 +250,6 @@ class CustomTestSuite(unittest.TestSuite):
         pass
 
 
-class CustomTestCase(unittest.TestCase):
-
-    compare = difflib.Differ().compare
-    """Comparison method shared by all subclasses."""
-
-    def __init__(self, method_name, input, expected, id,
-                 run_in_debugger=0, short_description=None):
-        """
-        Initialise the CustomTestCase.
-
-        Arguments:
-
-        method_name -- name of test method to run.
-        input -- input to the parser.
-        expected -- expected output from the parser.
-        id -- unique test identifier, used by the test framework.
-        run_in_debugger -- if true, run this test under the pdb debugger.
-        short_description -- override to default test description.
-        """
-        self.id = id
-        self.input = input
-        self.expected = expected
-        self.run_in_debugger = run_in_debugger
-        # Ring your mother.
-        unittest.TestCase.__init__(self, method_name)
-
-    def __str__(self):
-        """
-        Return string conversion. Overridden to give test id, in addition to
-        method name.
-        """
-        return '%s; %s' % (self.id, unittest.TestCase.__str__(self))
-
-    def __repr__(self):
-        return "<%s %s>" % (self.id, unittest.TestCase.__repr__(self))
-
-    def compare_output(self, input, output, expected):
-        """`input`, `output`, and `expected` should all be strings."""
-        if type(input) == UnicodeType:
-            input = input.encode('raw_unicode_escape')
-        if type(output) == UnicodeType:
-            output = output.encode('raw_unicode_escape')
-        if type(expected) == UnicodeType:
-            expected = expected.encode('raw_unicode_escape')
-        try:
-            self.assertEquals('\n' + output, '\n' + expected)
-        except AssertionError:
-            print >>sys.stderr, '\n%s\ninput:' % (self,)
-            print >>sys.stderr, input
-            print >>sys.stderr, '-: expected\n+: output'
-            print >>sys.stderr, ''.join(self.compare(expected.splitlines(1),
-                                                     output.splitlines(1)))
-            raise
-
-    def skip_test(self):
-        print >>sys.stderr, '%s: Test skipped' % self
-
-
-class TransformTestSuite(CustomTestSuite):
-
-    """
-    A collection of TransformTestCases.
-
-    A TransformTestSuite instance manufactures TransformTestCases,
-    keeps track of them, and provides a shared test fixture (a-la
-    setUp and tearDown).
-    """
-
-    def __init__(self, parser):
-        self.parser = parser
-        """Parser shared by all test cases."""
-
-        CustomTestSuite.__init__(self)
-
-    def generateTests(self, dict, dictname='totest',
-                      testmethod='test_transforms'):
-        """
-        Stock the suite with test cases generated from a test data dictionary.
-
-        Each dictionary key (test type's name) maps to a list of transform
-        classes and list of tests. Each test is a list: input, expected
-        output, optional modifier. The optional third entry, a behavior
-        modifier, can be 0 (temporarily disable this test) or 1 (run this test
-        under the pdb debugger). Tests should be self-documenting and not
-        require external comments.
-        """
-        for name, (transforms, cases) in dict.items():
-            for casenum in range(len(cases)):
-                case = cases[casenum]
-                run_in_debugger = 0
-                if len(case)==3:
-                    if case[2]:
-                        run_in_debugger = 1
-                    else:
-                        continue
-                self.addTestCase(
-                      TransformTestCase, testmethod,
-                      transforms=transforms, parser=self.parser,
-                      input=case[0], expected=case[1],
-                      id='%s[%r][%s]' % (dictname, name, casenum),
-                      run_in_debugger=run_in_debugger)
-
-
 class TransformTestCase(CustomTestCase):
 
     """
@@ -324,6 +313,52 @@ class TransformTestCase(CustomTestCase):
         self.compare_output(self.input, output, self.expected)
 
 
+class TransformTestSuite(CustomTestSuite):
+
+    """
+    A collection of TransformTestCases.
+
+    A TransformTestSuite instance manufactures TransformTestCases,
+    keeps track of them, and provides a shared test fixture (a-la
+    setUp and tearDown).
+    """
+
+    def __init__(self, parser):
+        self.parser = parser
+        """Parser shared by all test cases."""
+
+        CustomTestSuite.__init__(self)
+
+    def generateTests(self, dict, dictname='totest',
+                      testmethod='test_transforms'):
+        """
+        Stock the suite with test cases generated from a test data dictionary.
+
+        Each dictionary key (test type's name) maps to a tuple, whose
+        first item is a list of transform classes and whose second
+        item is a list of tests. Each test is a list: input, expected
+        output, optional modifier. The optional third entry, a
+        behavior modifier, can be 0 (temporarily disable this test) or
+        1 (run this test under the pdb debugger). Tests should be
+        self-documenting and not require external comments.
+        """
+        for name, (transforms, cases) in dict.items():
+            for casenum in range(len(cases)):
+                case = cases[casenum]
+                run_in_debugger = 0
+                if len(case)==3:
+                    if case[2]:
+                        run_in_debugger = 1
+                    else:
+                        continue
+                self.addTestCase(
+                      TransformTestCase, testmethod,
+                      transforms=transforms, parser=self.parser,
+                      input=case[0], expected=case[1],
+                      id='%s[%r][%s]' % (dictname, name, casenum),
+                      run_in_debugger=run_in_debugger)
+
+
 class ParserTestCase(CustomTestCase):
 
     """
@@ -396,7 +431,7 @@ class PEPParserTestCase(ParserTestCase):
 
     """PEP-specific parser test case."""
 
-    parser = rst.Parser(rfc2822=1, inliner=pep.Inliner())
+    parser = rst.Parser(rfc2822=1, inliner=rst.states.Inliner())
     """Parser shared by all PEPParserTestCases."""
 
     option_parser = frontend.OptionParser(components=(rst.Parser, pep.Reader))
@@ -586,7 +621,7 @@ class WriterPublishTestCase(CustomTestCase, docutils.SettingsSpec):
     Test case for publish.
     """
 
-    settings_default_overrides = {'_disable_config': 1}
+    settings_default_overrides = {'_disable_config': 1, 'strict_visitor': 1}
     writer_name = '' # override in subclasses
 
     def test_publish(self):
@@ -602,20 +637,12 @@ class WriterPublishTestCase(CustomTestCase, docutils.SettingsSpec):
 
 
 class LatexWriterPublishTestCase(WriterPublishTestCase):
-
-    """
-    Test case for Latex writer.
-    """
-
+    """Test case for Latex writer."""
     writer_name = 'latex'
 
 
 class PseudoXMLWriterPublishTestCase(WriterPublishTestCase):
-
-    """
-    Test case for pseudo-XML writer.
-    """
-
+    """Test case for pseudo-XML writer."""
     writer_name = 'pseudoxml'
 
 
@@ -743,3 +770,46 @@ def exception_data(code):
     except Exception, detail:
         return (detail, detail.args,
                 '%s: %s' % (detail.__class__.__name__, detail))
+
+
+def _format_str(*args):
+    r"""
+    Return a tuple containing representations of all args.
+
+    Same as map(repr, args) except that it returns multi-line
+    representations for strings containing newlines, e.g.::
+
+        '''\
+        foo  \n\
+        bar
+
+        baz'''
+
+    instead of::
+
+        'foo  \nbar\n\nbaz'
+
+    This is a helper function for CustomTestCase.
+    """
+    import re
+    return_tuple = []
+    for i in args:
+        r = repr(i)
+        if '\n' in i and (isinstance(i, StringType) or
+                          isinstance(i, UnicodeType)):
+            stripped = ''
+            if isinstance(i, UnicodeType):
+                # stripped = 'u' or 'U'
+                stripped = r[0]
+                r = r[1:]
+            # quote_char = "'" or '"'
+            quote_char = r[0]
+            assert quote_char in ("'", "'")
+            assert r[0] == r[-1]
+            r = r[1:-1]
+            r = (stripped + 3 * quote_char + '\\\n' +
+                 re.sub(r'(?<!\\)((\\\\)*)\\n', r'\1\n', r) +
+                 3 * quote_char)
+            r = re.sub(r' \n', r' \\n\\\n', r)
+        return_tuple.append(r)
+    return tuple(return_tuple)
