@@ -2,8 +2,8 @@
 
 # Author: David Goodger
 # Contact: goodger@users.sourceforge.net
-# Revision: $Revision: 1.15 $
-# Date: $Date: 2003/06/29 15:39:51 $
+# Revision: $Revision: 1.17 $
+# Date: $Date: 2004/10/30 16:02:08 $
 # Copyright: This module has been placed in the public domain.
 
 """Tests for states.py."""
@@ -250,21 +250,12 @@ Test missing titles; blank line in-between.
 """,
 """\
 <document source="test data">
-    <system_message level="3" line="1" source="test data" type="ERROR">
-        <paragraph>
-            Document or section may not begin with a transition.
     <transition>
-    <system_message level="3" line="3" source="test data" type="ERROR">
-        <paragraph>
-            At least one body element must separate transitions; adjacent transitions not allowed.
     <transition>
     <paragraph>
         Test missing titles; blank line in-between.
     <transition>
     <transition>
-    <system_message level="3" line="9" source="test data" type="ERROR">
-        <paragraph>
-            Document or section may not end with a transition.
 """],
 ["""\
 ========================
@@ -862,9 +853,6 @@ Empty Section
     <section id="empty-section" name="empty section">
         <title>
             Empty Section
-        <system_message level="3" line="2" source="test data" type="ERROR">
-            <paragraph>
-                Section empty; must have contents.
 """],
 ["""\
 ===
@@ -900,9 +888,6 @@ Without it, the parser ends up in an infinite loop.
 """,
 """\
 <document source="test data">
-    <system_message level="3" line="0" source="test data" type="ERROR">
-        <paragraph>
-            Document empty; must have contents.
 """],
 ]
 
